/*----------------------------------------------------------------------------------------------

	Introduction to SHARC Multiprocessing Systems Using VisualDSP++.
	M.K.B., European DSP Applications.
	November 2001.

	ID1.ASM

	This program demonstrates some of the Multiprocessing features handled by VisualDSP++ Tools:

	1. Master Interprocessor DMA -> transfer from ID1 to ID2
	2. Master DMA -> transfer from SBSRAM to ID1
	3. Broadcast write -> broadcast transfer from ID1 to both ID1 & ID2
	4. Serves VIRPT generated by ID2
	5. Message Passing between DSPs (MSRGx)
	6. Bus locking -> preventing external decives from accessing data
	7. Also, an ID check routine has been added to ensure correct functionality.

-----------------------------------------------------------------------------------------------*/

#include	"def21160.h"

#define N 10
#define MMS_ID2	0x00200000			// Offset value for MMS Space ID2
#define MMS_Broadcast 0x74DB00		// Offset value for MMS Broadcast Space + 0x4DB00(int.mem.)

.extern ext_mem_data;				// Ext. mem. data buffer

.section/dm seg_dmda;
.align 2;		
.var 	Broadcast_data[N] = "broadcast_data.dat";		// Source for broadcast transfer to ID1 & ID2
.var	DMA_source_ID1[N] = "ID1_data.dat";				// Source for DMA transfer from ID1 to ID2
.var	DMA_dest_ID1[N];								// Destination for DMA transfer from SBSRAM to ID1

.section/pm seg_pmda;
.align 2;
.var	Broadcast_dest[N];								// Destination for broadcast transfer in ID1
.var    DAG_dest_ID1[N];								// Destination for DAG transfer from ID2 to ID1
.global DAG_dest_ID1;

.section/PM     seg_rth;

	nop; nop; nop; nop;			/* EMUI	  emulator interrupt */
	nop; jump start_ID1;rti;rti;/* RSTI   reset vector */
	nop; nop; nop; nop;			/* IICDI  illegal input cond. detected */
	rti; rti; rti; rti;			/* SOVFI  stack overflow */
	rti; rti; rti; rti;			/* TMZHI  Timer=0 high priority */
	rti; rti; rti; rti;			/* VIRPTI vector interrupt */
	rti; rti; rti; rti;			/* IRQ2I  irq2 vector */
	rti; rti; rti; rti;			/* IRQ1I  irq1 vector */
	rti; rti; rti; rti;			/* IRQ0I  irq0 vector */
	nop; nop; nop; nop;			/*   reserved */
	rti; rti; rti; rti;			/* SPR0I  DMA0 SP0_RX */
	rti; rti; rti; rti;			/* SPR1I  DMA1 SP1_RX */
	rti; rti; rti; rti;			/* SPT0I  DMA2 SP0_TX */
	rti; rti; rti; rti;			/* SPT1I  DMA3 SP1_TX */
	nop; nop; nop; nop;			/* LP0I	  DMA4 link buffer 0 */
	nop; nop; nop; nop;			/* LP1I   DMA5 link buffer 1 */
	rti; rti; rti; rti;			/* LP2I   DMA6 link buffer 2 */
	rti; rti; rti; rti;			/* LP3I   DMA7 link buffer 3 */
	rti; rti; rti; rti;			/* LP4I   DMA8 link buffer 4 */
	rti; rti; rti; rti;			/* LP5I   DMA9 link buffer 5 */
	jump DMA_10; rti; rti; rti;	// EP0I	  DMA10 EPB0	 
	jump DMA_11; rti; rti; rti;	// EP1I	  DMA11 EPB1	 
	rti; rti; rti; rti;			/* EP2I	  DMA12 EPB2	 */
	rti; rti; rti; rti;			/* EP3I	  DMA13 EPB3	 */
	rti; rti; rti; rti;			/* LSRQI  link port serive request */
	rti; rti; rti; rti;			/* CB7I	  circular buffer 7 overflow */
	rti; rti; rti; rti;			/* CB15I	  circular buffer 15 overflow */
	rti; rti; rti; rti;			/* TMZLI  Timer0=0 low priority */
	rti; rti; rti; rti;			/* FIXI	  fixed point overflow */
	rti; rti; rti; rti;			/* FLTOI  floating point overflow execption */
	rti; rti; rti; rti;			/* FLTUI  floating point underflow execption  */
	nop; nop; nop; nop;			/* FLTII  floating point invalid */
	R0=0x2f2f2f2f;				// SFT0I  user software interrupt 
	RTI (DB);					// Serve VIRPTI generated by SHARC ID2
	B0=MSGR0;					// Load address of MSGR 0 register and 
	DM(MMS_ID2,I0)=R0;			// write test value in SHARC ID 2 MSGR0 
	rti; rti; rti; rti;			/* SFT1I  user software interrupt  */  
	rti; rti; rti; rti;			/* SFT2I  user software interrupt  */   
	rti; rti; rti; rti;			/* SFT3I  user software interrupt  */ 
	rti; rti; rti; rti;			/* -      reserved lowest priority    */


.section/pm seg_pmco;

start_ID1:		
					ustat1=0x1CE7385;			// Initialize External Memory (in this case SBSRAM)
					dm(WAIT)=ustat1;			// synchr. access 2 cycles read and write
												// NOTE!!! all DSPs in the system MUST initialize the SBSRAM and with the same value
				
												// Check that code is being loaded in the right DSP (ID1)
					R0=DM(SYSTAT);				// get SYSTAT value 
					R1=FEXT R0 BY 8:3;			// get the IDC value 
					R2=0x1;						// ID=1
					R1=R1-R2;					// is this DSP ID1?
					IF NE JUMP incorrect_ID;	// if not stop execution and enter endless loop 
 				
				    BIT SET IMASK EP0I;	 		// Interrupt EP0 enable
   	  				BIT SET IMASK EP1I;	 		// Interrupt EP1 enable
					BIT SET IMASK VIRPTI;		// Vector processor interrupt enable
	     		    BIT SET MODE1 IRPTEN;	 	// Global Interrupt Enable


//	1. Perform first data transfer: Master DMA, DMA channel 11, transfer from ID1 to ID2

					r0=0;					dm(DMAC11)=r0;  // clear DMA Control Reg.
					r0=DMA_source_ID1;		dm(II11)=r0;	// source ID1
					r0=1;					dm(IM11)=r0;	// modifier = 1
					r0=10;					dm(C11)=r0;		// counter = 10
					r0=0x200006;			dm(EI11)=r0;	// write to slave EPB1 of ID2	 
					r0=0;					dm(EM11)=r0;	// no external modifier
					r0=10;					dm(EC11)=r0;	// external counter = 10
					
					ustat1=0x0404;			dm(DMAC11)=ustat1;	// transmit data, PMODE=100	
					ustat1=dm(DMAC11);		bit set ustat1 0x1;	// enable DMA channel
					dm(DMAC11)=ustat1;	

//  2. Perform second data transfer: Master DMA, DMA channel 10, transfer from SBSRAM to ID1

					r0=0;					dm(DMAC10)=r0;	// clear DMA Control Reg.
					r0=ext_mem_data;		dm(EI10)=r0;	// source SBSRAM
					r0=1;					dm(EM10)=r0;	// modifier = 1
					r0=10;					dm(EC10)=r0;	// counter = 1
					r0=DMA_dest_ID1;		dm(II10)=r0;	// destination in internal memory of ID1	 
					r0=1;					dm(IM10)=r0;	// modifier = 1
					r0=10;					dm(C10)=r0;		// counter = 10

					ustat1=0x0400;			dm(DMAC10)=ustat1;	// receive data, PMODE=100	
					ustat1=dm(DMAC10);		bit set ustat1 0x1;	// enable DMA channel
					dm(DMAC10)=ustat1;								


// 3. Perform a BROADCAST write to ALL DSPs in the system (ID1 & ID2)
										
												// Lock bus before performing broadcast write in order to prevent external
					BIT SET MODE2 BUSLK;		// devices from accessing data while being modified
					if NOT BM jump(PC,0);		// Wait until ID1 becomes bus master

					ustat1 = DM(SYSTAT);		// Get SYSTAT value 
					BIT TST USTAT1 12; 			// Test DWPD bit(indicates if a direct write to DSP's internal memory is pending) 
					if TF jump(PC,-2);			// If pending, wait.
			
					b1 = Broadcast_data; 	l1 = N;	m1 = 1;		// Source located in ID1 internal memory			
					b8 = MMS_Broadcast;		l8 = N;	m8 = 1;		// Destination in Broadcast MMS Space

					r2 = dm(i1,m1);						// Fetch first value
					lcntr = N-1;
					do broadcast_transfer until lce;
broadcast_transfer:	r2 = dm(i1,m1) , pm(i8,m8) = r2;	// Dual access in 1 cycle.	
					pm(i8,m8) = r2;						// Load last value
	
					BIT CLR MODE2 BUSLK;		// Free the bus after broadcast data transfer has completed.

							
wait:				idle;						// wait forever
					jump wait;

	
DMA_10:				nop; nop; rti;				// Dummy Interrupt service routine for DMA EPB0 - Return to interrupt.
DMA_11:				nop; nop; rti;				// Dummy Interrupt service routine for DMA EPB1 - Return to interrupt.

incorrect_ID:	   idle; jump incorrect_ID;  	// ERROR - Code loaded into wrong DSP. Check DSP ID Number!!! 





