
/*--------------------------------------------------------------------------------------------

	Introduction to SHARC Multiprocessing Systems Using VisualDSP++.
	M.K.B., European DSP Applications 
	November 2001.

	ID2.ASM

	This program demonstrates some of the Multiprocessing features handled by VisualDSP++ Tools:

	1. Slave Interprocessor DMA -> transfer from ID1 to ID2
	2. DAG transfer -> data transfer from ID2 to ID1
	3. Generates VIRPT Interrupt in ID1
	4. Also, an ID check routine has been added to ensure correct functionality.


*--------------------------------------------------------------------------------------------*/

#include "def21160.h"

#define N 10

.extern DAG_dest_ID1;

.section/dm seg_dmda;
.align 2;
.var		DMA_dest_ID2[N];
.global		DMA_dest_ID2;
.var		DAG_source_ID2[N]="ID2_data.dat";
.global		DAG_source_ID2;

.section/pm seg_pmda;
.align 2;
.var 		Broadcast_dest[N];

.SECTION/PM     seg_rth;

	nop; nop; nop; nop;			/* EMUI	  emulator interrupt */
	nop; jump start_ID2;rti;rti;/* RSTI   reset vector */
	nop; nop; nop; nop;			/* IICDI  illegal input cond. detected */
	rti; rti; rti; rti;			/* SOVFI  stack overflow */
	rti; rti; rti; rti;			/* TMZHI  Timer=0 high priority */
	rti; rti; rti; rti;			/* VIRPTI vector interrupt */
	rti; rti; rti; rti;			/* IRQ2I  irq2 vector */
	rti; rti; rti; rti;			/* IRQ1I  irq1 vector */
	rti; rti; rti; rti;			/* IRQ0I  irq0 vector */
	nop; nop; nop; nop;			/* Reserved */
	rti; rti; rti; rti;			/* SPR0I  DMA0 SP0_RX */
	rti; rti; rti; rti;			/* SPR1I  DMA1 SP1_RX */
	rti; rti; rti; rti;			/* SPT0I  DMA2 SP0_TX */
	rti; rti; rti; rti;			/* SPT1I  DMA3 SP1_TX */
	nop; nop; nop; nop;			/* LP0I	  DMA4 link buffer 0 */
	nop; nop; nop; nop;			/* LP1I   DMA5 link buffer 1 */
	rti; rti; rti; rti;			/* LP2I   DMA6 link buffer 2 */
	rti; rti; rti; rti;			/* LP3I   DMA7 link buffer 3 */
	rti; rti; rti; rti;			/* LP4I   DMA8 link buffer 4 */
	rti; rti; rti; rti;			/* LP5I   DMA9 link buffer 5 */
	rti; rti; rti; rti;			/* EP0I	  DMA10 EPB0	 */
	jump DMA_11; rti; rti; rti;	// EP1I	  DMA11 EPB1	 
	rti; rti; rti; rti;			/* EP2I	  DMA12 EPB2	 */
	rti; rti; rti; rti;			/* EP3I	  DMA13 EPB3	 */
	rti; rti; rti; rti;			/* LSRQI  link port serive request */
	rti; rti; rti; rti;			/* CB7I	  circular buffer 7 overflow */
	rti; rti; rti; rti;			/* CB15I	  circular buffer 15 overflow */
	rti; rti; rti; rti;			/* TMZLI  Timer0=0 low priority */
	rti; rti; rti; rti;			/* FIXI	  fixed point overflow */
	rti; rti; rti; rti;			/* FLTOI  floating point overflow execption */
	rti; rti; rti; rti;			/* FLTUI  floating point underflow execption  */
	nop; nop; nop; nop;			/* FLTII  floating point invalid */
	rti; rti; rti; rti;			/* SFT0I  user software interrupt  */ 
	rti; rti; rti; rti;			/* SFT1I  user software interrupt  */  
	rti; rti; rti; rti;			/* SFT2I  user software interrupt  */   
	rti; rti; rti; rti;			/* SFT3I  user software interrupt  */ 
	rti; rti; rti; rti;			/* -   Reserved lowest priority    */


.section/pm seg_pmco;

start_ID2:	
					ustat1=0x1CE7385;			// Initialize External Memory (in this case SBSRAM)
					dm(WAIT)=ustat1;			// synchr. access 2 cycles read and write 
												// NOTE!!! all DSPs in the system MUST initialize the SBSRAM and with the same value

												// Check that code is being loaded in the right DSP (ID2)
					R0=DM(SYSTAT);				// get SYSTAT value 
					R1=FEXT R0 BY 8:3;			// get the IDC value 
					R2=0x2;						// ID=2
					R1=R1-R2;					// is this DSP ID2?
					IF NE JUMP incorrect_ID;	// if not stop execution and enter endless loop 

					BIT SET IMASK EP1I;	 		// Interrupt EP1 enable 
   				    BIT SET MODE1 IRPTEN;	 	// Global Interrupt Enable 


//	1. Perform first data transfer: Slave DMA, DMA channel 11, receive data transmitted from ID1

					r0=0;					dm(DMAC11)=r0;	// clear DMA Control Reg.
					r0=DMA_dest_ID2;		dm(II11)=r0;	// destination ID2
					r0=1;					dm(IM11)=r0;	// modifier = 1
					r0=10;					dm(C11)=r0;		// counter = 10
					r0=0x01;				dm(DMAC11)=r0;	// enable to receive data 

	
//  2. Perform second data transfer: DAG transfer from ID2's internal memory to ID1's internal memory.

					b1 = DAG_source_ID2; 		l1 = N;	m1 = 1;		// Source in ID2
					b8 = DAG_dest_ID1; 			l8 = N;	m8 = 1;		// Destination in ID1	
					
					
					r2 = dm(i1,m1);							// Fetch first value
					lcntr = N-1, do DAG_transfer until lce;
DAG_transfer:		r2 = dm(i1,m1), pm(i8,m8) = r2;			// Dual access in 1 cycle.
					pm(i8,m8) = r2;							// Load last value

//	3. Generate VIRPT Interrupt		

					B0=0x100001;				// IOP VIRPT register address in ID1 
					R0=0x40080;					// ISR at SFT0I address in ID1 will be executed 
					DM(I0,M0)=R0;				// Execute VIRPT interrupt in ID1 

wait:				idle;						// wait forever
					jump wait;

DMA_11:				nop; nop; rti;				// Dummy Interrupt service routine for DMA EPB1 - Return to interrupt.

incorrect_ID:   	idle; jump incorrect_ID;  	// ERROR - Code loaded into wrong DSP. Check DSP ID Number!!! 
